//vstupne parametre (treba dat na stdin): typ_vstupu n m max_s max_degree
//typ_vstupu:
//  0 - random
//  1 - riesenie zarucene existuje
//  2 - riesenie zarucene neexistuje
//  3 - jeden vrchol je velkeho stupna (hviezda)
//  4 - jeden vrchol ma velky stupen a po kazdej hrane treba prejst
//  5 - cesta z $a$ do $b$ ide cez vsetky vrcholy aspon raz
//
// Ak sa pre dane vstupne parametre neda vyrobit pozadovany graf, generator by sa mal zacyklit

#include <bits/stdc++.h>
#include <sys/types.h>
#include <unistd.h>
using namespace std;

#define inf 1023456789
#define linf 1023456789123456789ll
#define pii pair<int,int>
#define pipii pair<int, pii >
#define pll pair<long long,long long>
#define vint vector<int>
#define vvint vector<vint >
#define ll long long
#define pdd pair<double, double>

#define DEBUG
#ifdef DEBUG
#define db(x) cerr << #x << " = " << x << endl
#else
#define db(x)
#endif

int generuj_k(int max_s) //generuje cislo z [0, max_s], pricom mensie cisla maju vacsiu sancu
{
	int pom = rand()%(max_s + 1);
	return rand()%(pom+1);
}

struct graf
{
	int n, m;
	struct hrana
	{
		int odkial, kam;
		int sirka;
	};
	vector<hrana> ulica;
	vector<vector<int> > sused_von, sused_dnu;
	
	graf(int v = 0)
	{
		n = v;
		m = 0;
		sused_dnu.resize(n);
		sused_von.resize(n);
	}
	
	bool pridaj_ulicu(int x, int y, int s, int max_degree)
	{
		if(x == y)return 0;
		if(sused_dnu[x].size() + sused_von[x].size() == max_degree || sused_dnu[y].size() + sused_von[y].size() == max_degree)
		{
			return 0;
		}
		hrana nova;
		nova.odkial = x;
		nova.kam = y;
		nova.sirka = s;
		sused_von[x].push_back(ulica.size());
		sused_dnu[y].push_back(ulica.size());
		ulica.push_back(nova);
		m++;
		return 1;
	}
	
	void pridaj_nahodnu_ulicu(int max_s, int max_degree)
	{
		int x, y, s;
		do
		{
			x = rand() % n;
			y = rand() % n;
			s = rand() % max_s + 1;
		} while(!pridaj_ulicu(x, y, s, max_degree));
	}
	
	void utried_hrany()
	{
		for(int i=0; i<n; i++)
		{
			vector<pii > pom;
			for(int j=0; j<sused_von[i].size(); j++)
			{
				int hid = sused_von[i][j];
				pom.push_back(pii(ulica[hid].sirka, hid));
			}
			sort(pom.begin(), pom.end());
			for(int j=0; j<sused_von[i].size(); j++)
			{
				sused_von[i][j] = pom[j].second;
			}
		}
	}
	
	bool moze_nadvazovat(int a, int b, int k)
	{
		return ulica[a].kam == ulica[b].odkial && ulica[a].sirka - k <= ulica[b].sirka;
	}
	
	vector<int> zoznam_dosiahnutelnych;
	vector<int> bol_som;
	vector<int> kolko_dam;
	
	void dfs(int hid, int k) //divny-first search
	{
		zoznam_dosiahnutelnych.push_back(hid);
		bol_som[hid] = 1;
		int vid = ulica[hid].kam;
		vint to_do;
		for(int i=sused_von[vid].size()-1 - kolko_dam[vid]; i>=0; i--)
		{
			int on = sused_von[vid][i];
			if(moze_nadvazovat(hid, on, k))
			{
				to_do.push_back(on);
				kolko_dam[vid]++;
			}
			else
			{
				break;
			}
		}
		for(int i=0; i<to_do.size(); i++)
		{
			dfs(to_do[i], k);
		}
	}
	
	void zaruc_riesitelnost(int a, int b, int k, int max_s, int max_degree)
	{
		utried_hrany();
		bol_som = vector<int> (m, 0);
		kolko_dam = vector<int> (n, 0);
		kolko_dam[a] = sused_von[a].size();
		for(int i=0; i<sused_von[a].size(); i++)
		{
			dfs(sused_von[a][i], k);
		}
		for(int i=0; i<sused_dnu[b].size(); i++)
		{
			if(bol_som[sused_dnu[b][i]])return;
		}
		int x, s;
		if(zoznam_dosiahnutelnych.size() == 0)
		{
			while(!pridaj_ulicu(a, b, rand() %max_s+1, max_degree));
			return;
		}
		do
		{
			int naviazem_na = zoznam_dosiahnutelnych[rand() % zoznam_dosiahnutelnych.size()];
			x = ulica[naviazem_na].kam;
			int mins = max(1, ulica[naviazem_na].sirka - k);
			s = rand() % (max_s+1-mins) + mins;
		} while(!pridaj_ulicu(x, b, s, max_degree));
	}
	
	void vypis_hrany()
	{
		random_shuffle(ulica.begin(), ulica.end());
		for(int i=0; i<m; i++)
		{
			printf("%d %d %d\n",ulica[i].odkial, ulica[i].kam, ulica[i].sirka);
		}
	}
};

void generuj_nahodny(int n, int m, int max_s, int max_degree)
{
	graf res(n);
	for(int i=0; i<m; i++)
	{
		res.pridaj_nahodnu_ulicu(max_s, max_degree);
	}
	int k = generuj_k(max_s);
	int a = rand()%n;
	int b;
	do
	{
		b = rand()%n;
	} while(b == a);
	printf("%d %d %d %d %d\n", n, m, a, b, k);
	res.vypis_hrany();
}

void generuj_riesitelny(int n, int m, int max_s, int max_degree)
{
	graf res(n);
	for(int i=0; i<m-1; i++)
	{
		res.pridaj_nahodnu_ulicu(max_s, max_degree);
	}
	int a = rand() % n;
	int b;
	do
	{
		b = rand() % n;
	} while(b == a);
	int k = generuj_k(max_s);
	res.zaruc_riesitelnost(a, b, k, max_s, max_degree);
	if(res.m < m)
	{
		res.pridaj_nahodnu_ulicu(max_s, max_degree);
	}
	printf("%d %d %d %d %d\n", n, m, a, b, k);
	res.vypis_hrany();
}

void generuj_neriesitelny(int n, int m, int max_s, int max_degree)
{
	graf res(n);
	int k = generuj_k(max_s);
	int a = rand()%n;
	int b;
	do
	{
		b = rand()%n;
	} while(a == b);
	vint min_sirka(n); //minimalna sirka davu ked pride do i-teho mesta
	for(int i=0; i<n; i++)
	{
		min_sirka[i] = rand() % max_s + 2;
	}
	min_sirka[a] = 0;
	min_sirka[b] = max_s + 1;
	
	for(int i=0; i<m; i++)
	{
		int x, y, s;
		do
		{
			do
			{
				x = rand() % n;
				y = rand() % n;
				s = rand() % max_s + 1;
			} while(s < min_sirka[y] && s >= min_sirka[x] - k);
		} while(!res.pridaj_ulicu(x, y, s, max_degree));
	}
	printf("%d %d %d %d %d\n", n, m, a, b, k);
	res.vypis_hrany();
}

void generuj_hviezdu(int n, int m, int max_s, int max_degree)
{
	graf res(n);
	int a, b, c;
	a = rand()%n;
	do
	{
		b = rand()%n;
	} while(a == b);
	do
	{
		c = rand()%n;
	} while(a == c || b == c);
	int k = generuj_k(max_s);
	for(int i=0; i<m; i++)
	{
		int x, y, s;
		do
		{
			x = rand() % n;
			y = c;
			if(rand()%2)swap(x,y);
			s = rand() % max_s + 1;
		} while(!res.pridaj_ulicu(x, y, s, max_degree));
	}
	printf("%d %d %d %d %d\n", n, m, a, b, k);
	res.vypis_hrany();
}


void generuj_kvietok(int n, int m, int max_s, int max_degree)
{
	graf res(n);
	int k = generuj_k(min(max_s/((m+1)*2), max_s-1)) + 1;
	int a, b, c;
	a = rand()%n;
	do
	{
		b = rand()%n;
	} while(a == b);
	do
	{
		c = rand()%n;
	} while(a == c || b == c);
	
	int od = a;
	int posledna = max_s, predposledna = inf;
	for(int i=0; i<m; i+=2)
	{
		int s;
		int malo = max(posledna - k, 1), vela = min(predposledna - k, posledna);
		if(vela - malo < 1)break;
		s = rand() %(vela - malo) + malo;
		if(i == m-1)
		{
			res.pridaj_ulicu(od, b, s, max_degree);
			break;
		}
		res.pridaj_ulicu(od, c, s, max_degree);
		predposledna = posledna;
		posledna = s;
		malo = max(posledna-k, 1);
		vela = min(predposledna - k, posledna);
		if(vela - malo < 1)break;
		s = rand() %(vela - malo) + malo;
		int kam;
		if(i == m-2)kam = b;
		else
		{
			do
			{
				kam = rand() % n;
			} while(kam == c || kam == b || kam == a);
		}
		res.pridaj_ulicu(c, kam, s, max_degree);
		predposledna = posledna;
		posledna = s;
		od = kam;
	}
	if(res.m != m) //zacyklenie, ak bol maly max_degree, alebo male max_s
	{
		db(res.m);
		while(1);
	}
	printf("%d %d %d %d %d\n", n, m, a, b, k);
	res.vypis_hrany();
}

void generuj_cestu(int n, int m, int max_s, int max_degree)
{
	int k = generuj_k(min(max_s/10+1, max_s));
	graf res(n);
	vector<int> perm(n);
	for(int i=0; i<n; i++)perm[i] = i;
	random_shuffle(perm.begin(), perm.end());
	
	vint ziskam_najmenej(n), nepomoze_mi(n);
	ziskam_najmenej[perm[0]] = 0;
	nepomoze_mi[perm.back()] = max_s+1;
	int predosle = (max_s+1)/2;
	for(int i=0; i<n-1; i++)
	{
		int s = predosle - k + rand()%(2*k+1);
		s = min(max(s, 1), max_s);
		res.pridaj_ulicu(perm[i], perm[i+1], s, max_degree);
		ziskam_najmenej[perm[i+1]] = predosle-k;
		nepomoze_mi[perm[i]] = s+k+1;
		predosle = s;
	}
	while(res.m < m)
	{
		int x, y, s;
		x = rand() % n;
		y = rand() % n;
		s = rand() % max_s;
		if(s >= ziskam_najmenej[x]-k && s < nepomoze_mi[y])continue;
		res.pridaj_ulicu(x, y, s, max_degree);
	}
	if(res.m != m)
	{
		while(1);
	}
	printf("%d %d %d %d %d\n", n, m, perm[0], perm.back(), k);
	res.vypis_hrany();
}

int main()
{
	srand(time(0) << 7 ^ getpid());
	int typ, n, m, max_s, max_degree;
	scanf("%d %d %d %d %d", &typ, &n, &m, &max_s, &max_degree);
	if(max_degree == -1)max_degree = m;
	switch(typ)
	{
		case 0:
		{
			generuj_nahodny(n, m, max_s, max_degree);
			break;
		}
		case 1:
		{
			generuj_riesitelny(n, m, max_s, max_degree);
			break;
		}
		case 2:
		{
			generuj_neriesitelny(n, m, max_s, max_degree);
			break;
		}
		case 3:
		{
			generuj_hviezdu(n, m, max_s, max_degree);
			break;
		}
		case 4:
		{
			generuj_kvietok(n, m, max_s, max_degree);
			break;
		}
		case 5:
		{
			generuj_cestu(n, m, max_s, max_degree);
			break;
		}
	};
	return 0;
}